dnl ---------------------------------------------------------------------
dnl  Check for Support of md5sum calculations
dnl ---------------------------------------------------------------------
AC_DEFUN([AC_LDAS_CHECK_LIB_FRAMEL],[
  AC_LANG_SAVE
  AC_LANG_C
  ldas_saved_cflags=$CFLAGS
  ldas_saved_libs=$LIBS
  AC_ARG_WITH([framel],
  [ --with-framel=DIR   Directory where libframe exists],,
  [ for dir in /opt/lscsoft/libframe $prefix `echo $LD_LIBRARY_PATH | sed -e 's/:/ /g'`
    do
      dir="`echo $dir | sed -e 's/\/lib$//'`";
      if test -f "$dir/include/FrameL.h"
      then
        with_framel="$dir"
	break
      fi
    done ])

  case x$with_framel in
  x)
    ;;
  *)
    LIBS="$LIBS -L$with_framel/lib"
    ;;
  esac
  AC_CHECK_LIB(Frame,FrVectCompress,[
    AC_DEFINE(HAVE_LIBFRAME,1,Define to 1 if you have the `Frame' library (-lFrame))
    case x$with_framel in
    x)
      ;;
    *)
      LIBFRAME="-L$with_framel/lib "
      CPPFLAGSFRAME="-I$with_framel/include"
      ;;
    esac
    LIBFRAME="${LIBFRAME}-lFrame"
    LIBS="${lt_prog_compiler_static} ${LIBFRAME} -lm"
    CFLAGS="$CPPFLAGSFRAME $CFLAGS"
    AC_RUN_IFELSE([
	#include "FrameL.h"

	extern int FrSlong;

	int
	main( )
	{
	  FrLibShortIni( );
	  /*
	   * Zero exit (true) if FR_LONG_LONG should be defined
	   * Non-zero (false) if FR_LONG_LONG should not be defined
	   */
	  exit( FrSlong == sizeof( long ) );
	}
    ],[
      AC_DEFINE([DEFINE_FR_LONG_LONG],1,[Define to 1 if FR_LONG_LONG needs to be defined])
    ],[],[])

  ],[],-lm)
  AC_SUBST(LIBFRAME)
  AC_SUBST(CPPFLAGSFRAME)
  AM_CONDITIONAL(HAVE_LIBFRAME, test "x$LIBFRAME" != "x")
  CFLAGS=$ldas_saved_cflags
  LIBS=$ldas_saved_libs
  AC_LANG_RESTORE
])

dnl ---------------------------------------------------------------------
dnl  Check for Support of md5sum calculations
dnl ---------------------------------------------------------------------
AC_DEFUN([LDAS_LIB_MD5SUM],
  [ dnl -----------------------------------------------------------------
    dnl  Check for the library first
    dnl -----------------------------------------------------------------
    case `uname` in
    Linux)

      AC_CHECK_LIB(crypto,MD5_Init,
	[ AC_DEFINE(HAVE_MD5_IN_CRYPTO,1,[Defined if have md5 routines in libcrypto])
	  LIBMD5="-lcrypto"
	  AC_SUBST(LIBMD5)
          AC_CHECK_HEADERS([openssl/md5.h])
	],[ AC_CHECK_LIB(md5,MD5Init,
	    [ AC_DEFINE(HAVE_LIBMD5,1,[Defined if have libmd5])
	      LIBMD5="-lmd5"
	      AC_SUBST(LIBMD5)
	      AC_CHECK_HEADERS([md5.h])
	      AC_CHECK_HEADERS([md5global.h])
            ],[AC_MSG_ERROR(Missing md5 support) ])])
      ;;
    *)
      AC_CHECK_HEADERS([CommonCrypto/CommonDigest.h])
      AC_CHECK_LIB(md5,MD5Init,
	[ AC_DEFINE(HAVE_LIBMD5,1,[Defined if have libmd5])
	  LIBMD5="-lmd5"
	  AC_SUBST(LIBMD5)
	  AC_CHECK_HEADERS([md5.h])
	  AC_CHECK_HEADERS([md5global.h])
	],[ AC_CHECK_LIB(crypto,MD5_Init,
	    [ AC_DEFINE(HAVE_MD5_IN_CRYPTO,1,[Defined if have md5 routines in libcrypto])
	      LIBMD5="-lcrypto"
	      AC_SUBST(LIBMD5)
              AC_CHECK_HEADERS([openssl/md5.h])
            ],[AC_MSG_ERROR(Missing md5 support) ])])
      ;;
    esac
  ])

dnl----------------------------------------------------------------------
dnl Check to see if ospace is available -
dnl variable:
dnl	$1 - a value other than "no" causes aborting of the configuration
dnl	     process. A value of "no" indicates that Object Space is
dnl	     an optional library.
dnl----------------------------------------------------------------------

AC_DEFUN([AC_LDAS_CHECK_LIB_OSPACE],
[ AC_CHECK_LIB(socket,connect)
  AC_CHECK_LIB(rt,sched_yield)
  AC_REQUIRE([AC_LDAS_CHECK_LIB_THREAD])
  AC_ARG_WITH(
	ospace, 
	[  --with-ospace[=DIR]         ObjectSpace directory],
	[ case x"$with_ospace" in
	  "x"|xno) ;;
	  *)
	    case x$enable_64bit in
            xyes)
	      # ---------------------------------------------------------
	      # Test for 64bit package configuration
	      # ---------------------------------------------------------
	      if test -f ${with_ospace}/lib/64/pkgconfig/ospace.pc
	      then
	        PKG_CONFIG_PATH="${with_ospace}/lib/64/pkgconfig:${PKG_CONFIG_PATH}"
		OSPACE_PKG_CONFIG="yes"
	      elsif test -f ${with_ospace}/lib64/pkgconfig/ospace.pc
	        PKG_CONFIG_PATH="${with_ospace}/lib64/pkgconfig:${PKG_CONFIG_PATH}"
		OSPACE_PKG_CONFIG="yes"
	      fi
              ;;
            *)
	      # ---------------------------------------------------------
	      # Test for 32bit package configuration
	      # ---------------------------------------------------------
	      if test -f ${with_ospace}/lib/pkgconfig/ospace.pc
	      then
	        PKG_CONFIG_PATH="${with_ospace}/lib/pkgconfig:${PKG_CONFIG_PATH}"
		OSPACE_PKG_CONFIG="yes"
	      fi
              ;;
            esac
	    export PKG_CONFIG_PATH
	    AC_MSG_CHECKING(for ospace pkgconfig)
	    AC_MSG_RESULT($OSPACE_PKG_CONFIG)
	    case x${OSPACE_PKG_CONFIG} in
            xyes)
	      # ---------------------------------------------------------
	      #  This version support pkgconfig querying
	      # ---------------------------------------------------------
	      CPPFLAGS="`pkg-config --cflags ospace` ${CPPFLAGS}"
	      CXXFLAGS="`pkg-config --cflags ospace` ${CXXFLAGS}"
	      LDFLAGS="-Wl,${RPATH},`pkg-config --libs-only-L ospace | sed 's/^ *-L//'` `pkg-config --libs-only-L ospace` ${LDFLAGS}"
	      EXTRA_LIBS="`pkg-config --libs-only-l ospace` ${EXTRA_LIBS}"
	      ospace_la="`pkg-config --libs-only-L ospace | sed -e 's/^ *-L//' | sed -e 's/ *$//'`/libospace.la"
              ;;
	    *)
	      # ---------------------------------------------------------
	      # Need to do it the old way
	      # ---------------------------------------------------------
	      case x$enable_64bit in
	      xyes)
	        if test -f "${with_ospace}/lib/64/libospace${STLDEBUG}.la"
	        then
	          ospace_la="${with_ospace}/lib/64/libospace${STLDEBUG}.la"
	        elif test -f "${with_ospace}/lib64/libospace${STLDEBUG}.la"
	        then
	          ospace_la="${with_ospace}/lib64/libospace${STLDEBUG}.la"
	        else
	          ospace_la="${with_ospace}/lib/libospace${STLDEBUG}.la"
	        fi
	        ;;
	      *)
	        ospace_la="${with_ospace}/lib/libospace${STLDEBUG}.la"
	        ;;
	      esac
	      AC_SUBST(ospace_la)
	      case x$enable_64bit in
	      xyes)
	        if test -f "${with_ospace}/include64/ospace/config2.h"
	        then
	          AC_LDAS_PREFIX_INCLUDE_PATH("${with_ospace}/include64")
	        else
	          AC_LDAS_PREFIX_INCLUDE_PATH("${with_ospace}/include")
	        fi
	        ;;
	      *)
	        AC_LDAS_PREFIX_INCLUDE_PATH("${with_ospace}/include")
	        ;;
	      esac
	      CPPFLAGS="$LDAS_DEFS $CPPFLAGS"
	      CXXFLAGS="$LDAS_DEFS $CXXFLAGS"
	      ;;
	    esac
	    ;;
	 esac
	])
	AC_CHECK_HEADERS( 
	  [ ospace/config2.h ],
	  [ AC_DEFINE( HAVE_LIBOSPACE,1,[Defined if Object Space should be used] ) EXTRA_LIBS="${EXTRA_LIBS} ${ospace_la}" ],
	  [  if test -n "$1" && test "$1" = "no"
	     then
		AC_MSG_WARN(Could not find ObjectSpace headers.)
             else
		AC_MSG_ERROR(Could not find ObjectSpace headers.)
	     fi
	  ])
	AC_SUBST(EXTRA_LIBS)
  	AM_CONDITIONAL(HAVE_OSPACE,test -n "${ospace_la}")
])

AC_DEFUN([AC_LDAS_CHECK_LIB_THREAD],[
  AC_ARG_WITH([threads],
	      [  --without-threads       Don't build reentrant code ],
       	      [ case x"$with_threads" in
                "x"|xno) ;;
                *)
		  CFLAGS="${CFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
		  CXXFLAGS="${CXXFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
		  LDAS_CHECK_PTHREAD_READ_WRITE_LOCK
		  ;;
	        esac
	      ],[
	        CFLAGS="${CFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
	        CXXFLAGS="${CXXFLAGS} -D_REENTRANT -D_POSIX_PTHREAD_SEMANTICS"
	        LDAS_CHECK_PTHREAD_READ_WRITE_LOCK
	      ])
])
dnl -------------------------------------------------------------------
dnl -- check for SGI STL library (to use instead of gcc provided one)
dnl -------------------------------------------------------------------
AC_DEFUN([LDAS_LIB_STLPORT],[
  dnl -----------------------------------------------------------------
  dnl --- STLport - Core library
  dnl -----------------------------------------------------------------
  AC_ARG_WITH([stlport],
  [ --with-stlport=DIR   Build with SGI STL library],,
  [ for dir in $prefix `echo $LD_LIBRARY_PATH | sed -e 's/:/ /g'`
    do
      dir="`echo $dir | sed -e 's/\/lib$//'`";
      if test -d "$dir/include/stlport"
      then
        with_stlport="$dir"
	break
      fi
    done ])
  AC_MSG_CHECKING(for stlport installation directory)
  case "x$with_stlport" in
  x|xno)
    AC_MSG_RESULT(none)
    ;;
  *)
    if test -d "$dir"/include/stlport
    then
      with_stlport="$dir"
      AC_MSG_RESULT($with_stlport/include/stlport)
      STLPORT_INCLUDE_DIR="$with_stlport/include/stlport"
      STLPORT_LIB_DIR="$with_stlport/lib"
      if test -e $STLPORT_LIB_DIR/libstlport_gcc.so
      then
        STLPORT_LIB="stlport_gcc"
        STLPORT_DEBUG_LIB="stlport_gcc_stldbg"
      else
        STLPORT_LIB="stlport"
        STLPORT_DEBUG_LIB="stlportg"
      fi
      STLPORT_DEBUG_CXXFLAGS="-D_STLP_DEBUG -D_STLP_DEBUG_ALLOC -D_STLP_DEBUG_UNINITIALIZED"
    else
      AC_MSG_ERROR( invalid stlport directory: $with_stlport )
    fi
    ;;
  esac
  dnl -------------------------------------------------------------------
  dnl -- check for debug STLport library
  dnl -------------------------------------------------------------------
  AC_ARG_ENABLE(stlport-debug,
  [ --enable-stlport-debug   Build with debug STL library],
  [enable_stlport_debug=yes],
  [enable_stlport_debug=no])
  dnl -------------------------------------------------------------------
  dnl -- Establish shell variable to know if using the debug version
  dnl -------------------------------------------------------------------
  case x"${with_stlport}" in
  x|xno)
    dnl -----------------------------------------------------------------
    dnl -- Either the installation directory was invalid or the user
    dnl --   specifically requested not to use STLport
    dnl -----------------------------------------------------------------
    ;;
  *)
    dnl -----------------------------------------------------------------
    dnl -- Want to use STLport
    dnl -----------------------------------------------------------------
    AC_MSG_CHECKING(for stlport debug)
    case x"${enable_stlport_debug}" in
    xyes)
      AC_MSG_RESULT(yes)
      case "${STLPORT_DEBUG_LIB}" in
        stlport_dbg)
        STLDEBUG="_dbg"
        ;;
      stlport_*_stldbg)
        STLDEBUG="_stldbg"
        ;;
      esac
      ;;
    *)
      dnl ---------------------------------------------------------------
      dnl Use non-debug version of stlport
      dnl ---------------------------------------------------------------
      AC_MSG_RESULT(no)
      ;;
    esac # x"${enable_stlport_debug}"
  esac # x"${enable_stlport}"
  dnl '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  dnl Set some compiler specific options
  dnl '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
  if test "$ldas_prog_cxx_vendor" = Sun
  then
    if test -z "$STLPORT_LIB"
    then
      CXXFLAGS="$CXXFLAGS -library=stlport4"
    else
      CXXFLAGS="$CXXFLAGS -library=no%Cstd"
    fi
  fi
  AC_SUBST(STLDEBUG)
]) dnl LDAS_LIB_STLPORT

dnl ---------------------------------------------------------------------
dnl   Check for Support of STLport Finish
dnl ---------------------------------------------------------------------
AC_DEFUN([LDAS_LIB_STLPORT_FINISH],[
  dnl -------------------------------------------------------------------
  dnl -- Establish compiler/linker flags for STLport
  dnl -------------------------------------------------------------------
  case x"${with_stlport}" in
  x|xno)
    dnl -----------------------------------------------------------------
    dnl -- Either the installation directory was invalid or the user
    dnl --   specifically requested not to use STLport
    dnl -----------------------------------------------------------------
    ;;
  *)
    dnl -----------------------------------------------------------------
    dnl -- Want to use STLport
    dnl -----------------------------------------------------------------
    case x"${enable_stlport_debug}" in
    xyes)
      debug_flags="$STLPORT_DEBUG_CXXFLAGS"
      stlport_lib="STLPORT_DEBUG_LIB";
      ;;
    *)
      dnl ---------------------------------------------------------------
      dnl Use non-debug version of stlport
      dnl ---------------------------------------------------------------
      stlport_lib="$STLPORT_LIB";
      ;;
    esac # x"${enable_stlport_debug}"
    case x${STLPORT_CXXFLAGS} in
    x) ;;
    *) AC_MSG_CHECKING(for stlport CXXFLAGS)
       CXXFLAGS="${CXXFLAGS} ${STLPORT_CXXFLAGS}"
       AC_MSG_RESULT(${STLPORT_CXXFLAGS})
       ;;
    esac
    case x${STLPORT_INCLUDE_DIR} in
    x) ;;
    *) AC_MSG_CHECKING(for stlport include flags)
       CXXFLAGS="${CXXFLAGS} -I${STLPORT_INCLUDE_DIR} $debug_flags"
       AC_MSG_RESULT(-I${STLPORT_INCLUDE_DIR} $debug_flags)
       ;;
    esac
    case x${STLPORT_LIB_DIR} in
    x)
       case x${stlport_lib} in
       x)
 	 ;;
       *)
         STLPORT_LDFLAGS="-l${stlport_lib}"
         ;;
       esac
       ;;
    *)
       STLPORT_LDFLAGS="${LDFLAGS} -L${STLPORT_LIB_DIR} -l${stlport_lib}"
       ;;
    esac
    case x$STLPORT_LDFLAGS in
    x) ;;
    *) AC_MSG_CHECKING(for stlport LDFLAGS)
       LDFLAGS="${LDFLAGS} ${STLPORT_LDFLAGS}"
       AC_MSG_RESULT(${STLPORT_LDFLAGS})
       ;;
    esac
    ;;
  esac # x"${enable_stlport}"
])
dnl ---------------------------------------------------------------------
dnl  Check for Support of md5sum calculations
dnl ---------------------------------------------------------------------
AC_DEFUN([LDAS_LIB_UMEM],
[
  dnl -------------------------------------------------------------------
  dnl  Detect if the OS supplies a version of libumem
  dnl -------------------------------------------------------------------
  case x$enable_64bit in
  xyes)
    for x in 64 \
	sparcv9
    do
      if test -f /usr/lib/$x/libumem.so -o -L /usr/lib/$x/libumem.so
      then
        LIBUMEM=/usr/lib/$x/libumem.so
	break
      fi
    done
    ;;
  *)
    if test -f /usr/lib/libumem.so -o -L /usr/lib/libumem.so
    then
      LIBUMEM=/usr/lib/libumem.so
    fi
    ;;
  esac
  AC_SUBST(LIBUMEM)
])
