#include "framecpp/config.h"

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"
#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/FrameSpec.tcc"

#include "framecpp/Version8/FrAdcData.hh"
#include "framecpp/Version8/FrSE.hh"
#include "framecpp/Version8/FrSH.hh"

#include "framecpp/Version8/PTR_STRUCT.hh"

using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;


using std::ceil;
using std::floor;

//-----------------------------------------------------------------------
// Local functions and variables
//-----------------------------------------------------------------------
static const FrameSpec::Info::frame_object_types s_object_id
= FrameSpec::Info::FSI_FR_ADC_DATA;

static INT_2U MAX_REF = 2;

namespace FrameCPP
{
  namespace Version_8
  {
    //-----------------------------------------------------------------------
    ///
    //-----------------------------------------------------------------------
    FrAdcDataNonPtr::
    FrAdcDataNonPtr( )
      :	bias( DEFAULT_BIAS ),
	slope( DEFAULT_SLOPE ),
	units( FR_ADC_DATA_DEFAULT_UNITS( ) ),
	timeOffset( DEFAULT_TIME_OFFSET ),
	fShift( DEFAULT_FSHIFT ),
	phase( DEFAULT_PHASE ),
	dataValid( DEFAULT_DATA_VALID )
    {
    }

#if WORKING
    //-----------------------------------------------------------------------
    ///
    //-----------------------------------------------------------------------
    FrAdcDataNonPtr::
    FrAdcDataNonPtr( Previous::FrAdcData& Source,
		     Common::IStream* Stream )
    {
      name = Source.GetName( );
      comment = Source.GetComment( );
      channelGroup = Source.GetChannelGroup( );
      channelNumber = Source.GetChannelNumber( );
      nBits = Source.GetNBits( );
      bias = Source.GetBias( );
      slope = Source.GetSlope( );
      units = Source.GetUnits( );
      sampleRate = Source.GetSampleRate( );
      timeOffset = Source.GetTimeOffset( );
      fShift = Source.GetFShift( );
      phase = Source.GetPhase( );
      dataValid = Source.GetDataValid( );
    }
#endif /* WORKING */

    //-----------------------------------------------------------------------
    //-----------------------------------------------------------------------
    FrAdcDataNonPtr::
    FrAdcDataNonPtr( Common::IStream& Stream )
    {
      Stream >> name
	     >> comment
	     >> channelGroup
	     >> channelNumber
	     >> nBits
	     >> bias
	     >> slope
	     >> units
	     >> sampleRate
	     >> timeOffset
	     >> fShift
	     >> phase
	     >> dataValid
	;
    }

    //-------------------------------------------------------------------
    /// \todo Inline this method
    //-------------------------------------------------------------------
    cmn_streamsize_type FrAdcDataNonPtr::
    bytes( const Common::StreamBase& Stream ) const
    {
      return
	name.Bytes( )
	+ comment.Bytes( )
	+ sizeof( channelGroup )
	+ sizeof( channelNumber )
	+ sizeof( nBits )
	+ sizeof( bias )
	+ sizeof( slope )
	+ units.Bytes( )
	+ sizeof( sampleRate )
	+ sizeof( timeOffset )
	+ sizeof( fShift )
	+ sizeof( phase )
	+ sizeof( dataValid )
	;
    }

    void FrAdcDataNonPtr::
    write( Common::OStream& Stream ) const
    {
      Stream << name
	     << comment
	     << channelGroup
	     << channelNumber
	     << nBits
	     << bias
	     << slope
	     << units
	     << sampleRate
	     << timeOffset
	     << fShift
	     << phase
	     << dataValid
	  ;
    }

    //-----------------------------------------------------------------------
    //
    //-----------------------------------------------------------------------

    const FrAdcDataNonPtr::bias_type		FrAdcDataNonPtr::DEFAULT_BIAS
    = FR_ADC_DATA_DEFAULT_BIAS;
    const FrAdcDataNonPtr::slope_type		FrAdcDataNonPtr::DEFAULT_SLOPE
    = FR_ADC_DATA_DEFAULT_SLOPE;
    const FrAdcDataNonPtr::fShift_type		FrAdcDataNonPtr::DEFAULT_FSHIFT
    = FR_ADC_DATA_DEFAULT_FSHIFT;
    const FrAdcDataNonPtr::timeOffset_type	FrAdcDataNonPtr::DEFAULT_TIME_OFFSET
    = FR_ADC_DATA_DEFAULT_TIME_OFFSET;
    const FrAdcDataNonPtr::dataValid_type	FrAdcDataNonPtr::DEFAULT_DATA_VALID
    = FR_ADC_DATA_DEFAULT_DATA_VALID;
    const FrAdcDataNonPtr::phase_type		FrAdcDataNonPtr::DEFAULT_PHASE
    = FR_ADC_DATA_DEFAULT_PHASE;

    FrAdcData::
    FrAdcData( )
      : object_type( s_object_id, StructDescription( ) )
    {
      name = "";
      channelGroup = channelGroup_type( 0 );
      channelNumber = channelNumber_type( 0 );
      nBits = 0;
      sampleRate = -1.0;
      bias = DEFAULT_BIAS;
      slope = DEFAULT_SLOPE;
      units = DEFAULT_UNITS( );
      fShift = DEFAULT_FSHIFT;
      timeOffset = DEFAULT_TIME_OFFSET;
      dataValid = DEFAULT_DATA_VALID;
      phase = DEFAULT_PHASE;
    }

    FrAdcData::
    FrAdcData( const FrAdcData& Source )
      : FrAdcDataNonPtr( Source ),
	FrAdcDataPS( Source ),
	object_type( s_object_id, StructDescription( ) ),
	Common::TOCInfo( Source )
    {
    }

    FrAdcData::
    FrAdcData( const name_type& Name,
	       channelGroup_type Group,
	       channelNumber_type Channel,
	       nBits_type NBits,
	       sampleRate_type SampleRate,
	       bias_type Bias,
	       slope_type Slope,
	       const units_type& Units,
	       fShift_type FShift,
	       timeOffset_type TimeOffset,
	       dataValid_type DataValid,
	       phase_type Phase )
      : object_type( s_object_id, StructDescription( ) )
    {
      name = Name;
      channelGroup = Group;
      channelNumber = Channel;
      nBits = NBits;
      sampleRate = SampleRate;
      bias = Bias;
      slope = Slope;
      units = Units;
      fShift = FShift;
      timeOffset = TimeOffset;
      dataValid = DataValid;
      phase = Phase;
    }

    FrAdcData::
    FrAdcData( Previous::FrAdcData& Source, istream_type* Stream )
      : 
#if WORKING
      FrAdcDataNonPtr( Source, Stream ),
      FrAdcDataPtrs< FrVect, FrVect >( Source, Stream ),
#endif /* WORKING */
	object_type( s_object_id, StructDescription( ) )
    {
      name = Source.GetName( );
      comment = Source.GetComment( );
      channelGroup = Source.GetChannelGroup( );
      channelNumber = Source.GetChannelNumber( );
      nBits = Source.GetNBits( );
      bias = Source.GetBias( );
      slope = Source.GetSlope( );
      units = Source.GetUnits( );
      sampleRate = Source.GetSampleRate( );
      timeOffset = Source.GetTimeOffset( );
      fShift = Source.GetFShift( );
      phase = Source.GetPhase( );
      dataValid = Source.GetDataValid( );

      if ( Stream )
      {
	//-------------------------------------------------------------------
	// Modify references
	//-------------------------------------------------------------------
	Stream->ReplaceRef( RefData( ), Source.RefData( ), MAX_REF );
	Stream->ReplaceRef( RefAux( ), Source.RefAux( ), MAX_REF );
      }
    }

    FrAdcData::
    FrAdcData( istream_type& Stream )
      : FrAdcDataNonPtr( Stream ),
	FrAdcDataPS( Stream ),
	object_type( s_object_id, StructDescription( ) )
    {
      Stream.Next( this );
    }

    FrAdcData::
    ~FrAdcData( )
    {
    }

    const std::string& FrAdcData::
    GetNameSlow() const
    {
      return GetName( );
    }

    FrAdcData& FrAdcData::
    Merge( const FrAdcData& RHS )
    {
      //:TODO: Need to implement Merge routine
      std::string msg( "Merge currently not implemented for " );
      msg += StructName( );

      throw std::domain_error( msg );
      return *this;
    }

    const char* FrAdcData::
    ObjectStructName( ) const
    {
      return StructName( );
    }


    const Description* FrAdcData::
    StructDescription( )
    {
      static Description ret;

      if ( ret.size( ) == 0 )
      {
	ret( FrSH( FrAdcData::StructName( ), s_object_id,
		   "AdcData  Data Structure" ) );

	ret( FrSE( "name", "STRING",
		   "" ) );
	ret( FrSE( "comment", "STRING",
		   "" ) );
	ret( FrSE( "channelGroup", "INT_4U",
		   "" ) );
	ret( FrSE( "channelNumber", "INT_4U",
		   "" ) );
	ret( FrSE( "nBits", "INT_4U",
		   "" ) );
	ret( FrSE( "bias", "REAL_4",
		   "" ) );
	ret( FrSE( "slope", "REAL_4",
		   "" ) );
	ret( FrSE( "units", "STRING",
		   "" ) );
	ret( FrSE( "sampleRate", "REAL_8",
		   "" ) );
	ret( FrSE( "timeOffset", "REAL_8",
		   "" ) );
	ret( FrSE( "fShift", "REAL_8",
		   "" ) );
	ret( FrSE( "phase", "REAL_4",
		   "" ) );
	ret( FrSE( "dataValid", "INT_2U",
		   "" ) );

	ret( FrSE( "data", PTR_STRUCT::Desc( FrVect::StructName( ) ),
		   ""  ) );
	ret( FrSE( "aux", PTR_STRUCT::Desc( FrVect::StructName( ) ),
		   ""  ) );

	ret( FrSE( "next",	PTR_STRUCT::Desc( FrAdcData::StructName( ) ),
		   ""  ) );

	ret( FrSE( "chkSum", CheckSumDataClass( ), CheckSumDataComment( ) ) );
      }

      return &ret;
    }

    FrAdcData::subset_ret_type FrAdcData::
    Subset( REAL_8 Offset, REAL_8 Dt ) const
    {
      //---------------------------------------------------------------------
      // Sanity checks
      //---------------------------------------------------------------------
      // Verify the existance of only a single FrVect component
      //---------------------------------------------------------------------
      if ( data.size( ) != 1 )
      {
	std::ostringstream	msg;

	msg << "METHOD: frameAPI::FrAdcData::SubFrAdcData: "
	    << "data vectors of length " << data.size( )
	    << " currently are not supported";
	throw std::logic_error( msg.str( ) );
      }
      INT_4U	num_elements = data[0]->GetDim( 0 ).GetNx( );
      //---------------------------------------------------------------------
      //:TODO: Ensure the dimension of the FrVect is 1
      //---------------------------------------------------------------------
      //---------------------------------------------------------------------
      //:TODO: Ensure the requested range is valid
      //---------------------------------------------------------------------
      if ( Offset > ( GetTimeOffset( ) +
		      ( num_elements / GetSampleRate( ) ) ) )
      {
	std::ostringstream msg;
	msg << "METHOD: frameAPI::FrAdcData::SubFrAdcData: "
	    << "The value for the parameter Offset (" << Offset
	    << ") is out of range ([0-"
	    << ( GetTimeOffset( ) +
		 ( num_elements / GetSampleRate( ) ) )
	    << "))";
	throw std::range_error( msg.str( ) );
      }
      //---------------------------------------------------------------------
      // Start moving of data
      //---------------------------------------------------------------------
      subset_ret_type	retval( new FrAdcData );

      // Copy in the header information
      retval->copy_core( *this );

      //:TODO: Get the slice of FrVect
      REAL_8	startSampleReal =
	( Offset * retval->GetSampleRate( ) ) -
	( retval->GetTimeOffset( ) * retval->GetSampleRate( ) );

      INT_4U	endSample =
	INT_4U( ceil ( startSampleReal +
		       ( retval->GetSampleRate( ) * Dt ) ) );
      INT_4U	startSample = ( startSampleReal > REAL_8( 0 ) )
	? INT_4U( floor( startSampleReal ) )
	: INT_4U( 0 );

      {
	General::SharedPtr< FrVect >
	  subvect( this->data[ 0 ]->
		   SubFrVect( startSample, endSample ).release( ) );

	retval->data.append( subvect );
      }

      //---------------------------------------------------------------------
      // Shift the offset
      //---------------------------------------------------------------------
      if ( startSampleReal > 0 )
      {
	// There is no gap
	retval->SetTimeOffset( 0 );
      }
      else
      {
	retval->SetTimeOffset( retval->GetTimeOffset( ) + Offset );
      }
      //---------------------------------------------------------------------
      // Return the new FrAdcData
      //---------------------------------------------------------------------
      return retval;
    }
   
    void FrAdcData::
#if WORKING_VIRTUAL_TOCQUERY
    TOCQuery( int InfoClass, ... ) const
#else /*  WORKING_VIRTUAL_TOCQUERY */
    vTOCQuery( int InfoClass, va_list vl ) const
#endif /*  WORKING_VIRTUAL_TOCQUERY */
    {
      using Common::TOCInfo;

#if WORKING_VIRTUAL_TOCQUERY
      va_list	vl;
      va_start( vl, InfoClass );
#endif /*  WORKING_VIRTUAL_TOCQUERY */

      while ( InfoClass != TOCInfo::IC_EOQ )
      {
	int data_type = va_arg( vl, int );
	switch( data_type )
	{
	case TOCInfo::DT_STRING_2:
	  {
	    STRING* data = va_arg( vl, STRING* );
	    switch( InfoClass )
	    {
	    case TOCInfo::IC_NAME:
	      *data = GetName( );
	      break;
	    default:
	      goto cleanup;
	      break;
	    }
	  }
	  break;
	case TOCInfo::DT_INT_2U:
	  {
	    INT_2U* data = va_arg( vl, INT_2U* );
	    switch( InfoClass )
	    {
	    case TOCInfo::IC_DATA_VALID:
	      *data = GetDataValid( );
	      break;
	    default:
	      goto cleanup;
	      break;
	    }
	  }
	  break;
	case TOCInfo::DT_INT_4U:
	  {
	    INT_4U* data = va_arg( vl, INT_4U* );
	    switch( InfoClass )
	    {
	    case TOCInfo::IC_CHANNEL_ID:
	      *data = GetChannelNumber( );
	      break;
	    case TOCInfo::IC_GROUP_ID:
	      *data = GetChannelGroup( );
	      break;
	    default:
	      goto cleanup;
	      break;
	    }
	  }
	  break;
	default:
	  // Stop processing
	  goto cleanup;
	}
	InfoClass = va_arg( vl, int );
      }
    cleanup:
#if WORKING_VIRTUAL_TOCQUERY
      va_end( vl )
#endif /*  WORKING_VIRTUAL_TOCQUERY */
	;
    }

    FrAdcData::demote_ret_type FrAdcData::
    demote( INT_2U Target,
	    demote_arg_type Obj,
	    istream_type* Stream ) const
    {
      if ( Target >= DATA_FORMAT_VERSION )
      {
	return Obj;
      }
      try
      {
	//-------------------------------------------------------------------
	// Copy non-reference information
	//-------------------------------------------------------------------

	// Do actual down conversion
	General::SharedPtr< Previous::FrAdcData >
	  retval( new Previous::FrAdcData( GetName( ), GetChannelGroup( ),
					   GetChannelNumber( ), GetNBits( ),
					   GetSampleRate( ),
					   GetBias( ),
					   GetSlope( ),
					   GetUnits( ),
					   GetFShift( ),
					   GetTimeOffset( ),
					   GetDataValid( ),
					   GetPhase( )
					   ) )
	  ;
	retval->AppendComment( GetComment( ) );

	if ( Stream )
	{
	  //-----------------------------------------------------------------
	  // Modify references
	  //-----------------------------------------------------------------
	  Stream->ReplaceRef( retval->RefData( ), RefData( ), MAX_REF );
	  Stream->ReplaceRef( retval->RefAux( ), RefAux( ), MAX_REF );
	}
	//-------------------------------------------------------------------
	// Return demoted object
	//-------------------------------------------------------------------
	return retval;
      }
      catch( ... )
      {
      }
      throw
	Unimplemented( "Object* FrAdcData::demote( Object* Obj ) const",
		       DATA_FORMAT_VERSION, __FILE__, __LINE__ );
    }

    FrAdcData::promote_ret_type FrAdcData::
    promote( INT_2U Target,
	     promote_arg_type Obj,
	     istream_type* Stream ) const
    {
      return Promote( Target, Obj, Stream );
    }

    FrameCPP::cmn_streamsize_type FrAdcData::
    pBytes( const Common::StreamBase& Stream ) const
    {
      return
	FrAdcDataNonPtr::bytes( Stream ) +
	Stream.PtrStructBytes( ) +	// aux
	Stream.PtrStructBytes( ) +	// data
	Stream.PtrStructBytes( )	// next
	;
    }

    FrAdcData* FrAdcData::
    pCreate( istream_type& Stream ) const
    {
      return new FrAdcData( Stream );
    }

    void FrAdcData::
    pWrite( ostream_type& Stream ) const
    {
      try
      {
	FrAdcDataNonPtr::write( Stream );
	references_type::write( Stream );
	WriteNext( Stream );
      }
      catch ( const std::exception& Exception )
      {
	std::cerr << "DEBUG: Error Writing FrAdcData: " << GetName( )
		  << " exception: " << Exception.what( )
		  << std::endl;
	throw; // rethrow
      }
    }

    bool FrAdcData::
    operator==( const Common::FrameSpec::Object& Obj ) const
    {
      return ( *this == Obj );
    }

  } //  namespace - Version_8
} // namespace - FrameCPP
